#ifndef _Application_H_
#define _Application_H_

#include "SDL.h"
#include <map>
#include <vector>
#include <functional>

namespace sc {
    namespace gui {
        using EventHandler = std::function<bool(const SDL_Event&)>;
        using RunLoopAction = void(*)();

        class Application {
            public:
                static Application& instance() { return app; }

                Application() = default;
                ~Application() = default;

                // prohibit copying and moving
                Application(const Application&) = delete;
                Application& operator=(const Application&) = delete;
                Application(Application&&) = delete;
                Application& operator=(Application&&) = delete;

                void run();
                void add_event_handler(EventHandler handler, SDL_EventType event);
                void add_run_loop_action(RunLoopAction action);
                void remove_run_loop_action(RunLoopAction action);

                int fps() const { return _fps; }
                void fps(int fps) { _fps = fps; }

            private:
                static Application app;

                std::map<SDL_EventType, std::vector<EventHandler>> _event_handlers;
                std::vector<RunLoopAction> _actions;
                int _fps {60};

                bool handle_event(const SDL_Event& event);
        };

        // convenience function
        Application& app() { return Application::instance(); }
    }
}

#endif // _Application_H_
